#!/usr/bin/env python

"""
RSVP instruction classes.

Copyright (C) 2007 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class Instruction:

    "A generic instruction."

    def __init__(self, attr=None):
        self.attr = attr

    def __repr__(self):
        if self.attr is not None:
            return "%s(%r)" % (self.__class__.__name__, self.attr)
        else:
            return "%s()" % self.__class__.__name__

# Instructions operating on the value stack.

class LoadConst(Instruction):       "Load the constant from the specified location."

# Access within an invocation frame.

class LoadName(Instruction):        "Load the object from the given local attribute."
class StoreName(Instruction):       "Store the object in the given local attribute."

# Access to address-relative data.

class MakeObject(Instruction):      "Make a new object."
# ... DropObject not defined:        Assume garbage collection.
class LoadAttr(Instruction):        "Load the object from the given attribute."
class StoreAttr(Instruction):       "Store the object in the given attribute."

# Access to invocation frames in preparation.

class MakeFrame(Instruction):       "Make a new invocation frame."
class ReserveFrame(Instruction):    "Reserve the given number of entries for the invocation frame."
class DropFrame(Instruction):       "Drop an invocation frame."
class StoreFrame(Instruction):      "Store an argument at the given frame location."
class StoreFrameIndex(Instruction): "Store an argument for the parameter with the given index."

# Invocation-related instructions.

class Jump(Instruction): pass
class LoadCallable(Instruction): pass
class Return(Instruction): pass

# vim: tabstop=4 expandtab shiftwidth=4
